local mod = MorgDKP2:NewModule("Waitlist")
local L = LibStub("AceLocale-3.0"):GetLocale("MorgDKP2")
local media = LibStub("LibSharedMedia-3.0", true)
local BC = LibStub("LibBabble-Class-3.0"):GetLookupTable()

local mdkp = MorgDKP2

--locals
local LL = LibStub("AceLocale-3.0"):NewLocale("Waitlist", "enUS", true)
	
if LL then
	LL["Waitlist"] = true
	LL["Enable"] = true
	LL["Waitlist allows members to whisper you for invites and to receive DKP even if not in the actual raid."] = true
	LL["Award DKP"] = true
	LL["Award DKP to current waitlist members"] = true
	LL["Waitlist reward"] = true
	LL["Minimum time to award DKP to a waitlist member."] = true
	LL["Waitlist start DKP"] = true
	LL["Amount of DKP to reward at raidstart."] = true
	LL["Waitlist run DKP"] = true
	LL["Amount of DKP to reward after members have been on the waitlist for the current reward time."] = true
	LL["Waitlist boss DKP"] = true
	LL["Amount of DKP to reward to waitlist members per bosskill."] = true
	LL["Waitlist DKP/time"] = true
	LL["Amount of DKP to reward to waitlist members when awarding regular members DKP/time. Note: only works if you are awarding DKP/time."] = true
	LL["Removed %s from wailist."] = true
	LL["Added you to the waitlist."] = true
	LL["Removed you from the waitlist."] = true
	LL["Added %s to the waitlist."] = true
	LL["Please log on to your main for an invite to the raid.  Whisper me when online."] = true
end

local LL = LibStub("AceLocale-3.0"):GetLocale("Waitlist")

mod.modName = LL["Waitlist"]
mod.modref = "Waitlist"

local db
local fmt = string.format
local tinsert = table.insert
local tsort   = table.sort
local tremove = table.remove
local tostring = tostring 
local lower = string.lower
local sub = string.sub
local upper = string.upper
local gsub = string.gsub
local floor = math.floor
local find = string.find
local Player = UnitName("player")

local defaults = { 
	profile = {
		waitstart = 0,
		waitrun = 0,
		waitaward = nil,
		waitotime = 0,
		waitboss = 0,
		waitrewardtime = 1800,
		Waitlist = {members = {}, join = {}}
		
	},
}

local options = {
		type = "group",
		handler = mod,
		name = LL["Waitlist"],
		desc = LL["Waitlist"],
		order = 310,
		args = {
			enable = {
				type = "toggle",
				name = LL["Enable"],
				desc = LL["Waitlist allows members to whisper you for invites and to receive DKP even if not in the actual raid."],
				get = function() return mdkp.db.profile.modules.Waitlist end,
				set = 	function(info, v) 
						mdkp.db.profile.modules.Waitlist = v
						if v then 
							mod:Enable()
							mod:debug("Enabled Waitlist")
						else
							mod:Disable()
							mdkp:debug("Disabled Waitlist")
						end
					end,
				order = 5
			},
			waitaward = {
				type = "toggle",
				name = LL["Award DKP"],
				desc = LL["Award DKP to current waitlist members"],
				get = function() return mod.db.profile.waitaward end,
				set = function(info, v) mod.db.profile.waitaward = v end,
				hidden = function() return not mdkp.db.profile.modules.Waitlist end,
				order = 100
			},
			waitrewardtime = {
				type = "range",
				name = LL["Waitlist reward"],
				desc = LL["Minimum time to award DKP to a waitlist member."],
				get = function() return floor(mod.db.profile.waitrewardtime / 60) end,
				set = function(info, v) mod.db.profile.waitrewardtime = floor(v * 60) end,
				min = 0, 
				max = 120, 
				step = 1,
				hidden = function() return not mdkp.db.profile.modules.Waitlist end,
				order = 105
			},
			waitstart = {
				type = "range",
				name = LL["Waitlist start DKP"],
				desc = LL["Amount of DKP to reward at raidstart."],
				get = function() return mod.db.profile.waitstart end,
				set = function(info, v) mod.db.profile.waitstart = v end,
				min = 0, 
				max = 500, 
				step = 1,
				hidden = function() return not mdkp.db.profile.modules.Waitlist end,
				order = 110
			},
			waitrun = {
				type = "range",
				name = LL["Waitlist run DKP"],
				desc = LL["Amount of DKP to reward after members have been on the waitlist for the current reward time."],
				get = function() return mod.db.profile.waitrun end,
				set = function(info, v) mod.db.profile.waitrun = v end,
				min = 0, 
				max = 500, 
				step = 1,
				hidden = function() return not mdkp.db.profile.modules.Waitlist end,
				order = 115
			},
			waitboss = {
				type = "range",
				name = LL["Waitlist boss DKP"],
				desc = LL["Amount of DKP to reward to waitlist members per bosskill."],
				get = function() return mod.db.profile.waitboss end,
				set = function(info, v) mod.db.profile.waitboss = v end,
				min = 0, 
				max = 500, 
				step = 1,
				hidden = function() return not mdkp.db.profile.modules.Waitlist end,
				order = 120
			},
			waitotime = {
				type = "range",
				name = LL["Waitlist DKP/time"],
				desc = LL["Amount of DKP to reward to waitlist members when awarding regular members DKP/time. Note: only works if you are awarding DKP/time."],
				get = function() return mod.db.profile.waitotime end,
				set = function(info, v) mod.db.profile.waitotime = v end,
				min = 0, 
				max = 500, 
				step = 1,
				hidden = function() return not mdkp.db.profile.modules.Waitlist end,
				order = 120
			},
		},
}

function mod:GetOptions()
	return options
end

function mod:IsDKP()
	return nil
end

function mod:OnInitialize()
	self.db = mdkp.db:RegisterNamespace("Waitlist", defaults)
	
end

function mod:OnEnable()
	db = mdkp.db.profile
	db.moduleON["Waitlist"] = true
	
end

function mod:OnDisable()
	db.moduleON["Waitlist"] = nil
						
end

function mod:AtRaidStart()
	self:debug("Raidstart waitlist")
	self.waitlistdone = {}
	
end

function mod:AtRaidEnd()
	self.db.profile.Waitlist = {members = {}, join = {}}
	
end

function mod:AtMemberAdded(name) 	
	local POOL = db.raid
	for _, waitname in pairs(self.db.profile.Waitlist.members) do
		if waitname == name then
			self:SubWait(name)	
		elseif db.eqDKP[POOL].Aliases[name] and db.eqDKP[POOL].Aliases[name] == waitname then
			self:SubWait(db.eqDKP[POOL].Aliases[name])
		end
	end
end

function mod:AtAttendanceCheck(value, eventid)
	self:debug("Waitlist attendance check")
	local db2 = self.db.profile
	if db2.waitrun == 0 or not self.waitlistdone then return true end
	local POOL = db.raid
	local raidnum = db.raidnum
	local waitrewardtime = db2.waitrewardtime
	local currtime = date("%m/%d/%y %H:%M:%S")
	
	--determine who needs awarded
	local temp = {}
	for _,  name in pairs(db2.Waitlist.members) do
		if not self.waitlistdone[name] then
			if self:TimeDiff(db2.Waitlist.join[name], currtime)  >= db2.waitrewardtime then	
				table.insert(temp, name)
				self:debug("Added "..name.." to waitlist run.") 
				self.waitlistdone[name] = true
			end
		end
	end
	
	--award if necessary
	if temp and #temp > 0 then
		if db2.waitrun == value then
		 	for _, name in pairs(temp) do
		 		table.insert(db.raidlog[raidnum].bosskills[eventid].attendees, name)
		 	end
		else 
			self:AwardWaitotimeDKP(db2.waitrun, POOL, db.raidlog[raidnum].bosskills[eventid].name, temp, "Group", "Group") 
		end
		self:AwardDKP(nil, db2.waitrun, POOL, temp, 1, 1)
	end
	
	--done with waitlist?
	local waitdone = true
	for _, name in pairs(db2.Waitlist.members) do
		if not self.waitlistdone[name] then waitdone = nil end
	end
	return waitdone
end

function mod:AwardWaitlistDKP(value, POOL, Boss, id, quiet)
	if #self.db.profile.Waitlist.members == 0 then return end
	if not id then
		local eventnum
		for num, data in pairs(db.raidlog[db.raidnum].bosskills) do
			if find(data.name, Boss) or find(data.note, Boss) then eventnum = num break end
		end
		if not eventnum then return end
		if self.db.profile.waitboss == 0 then return end
		if self.db.profile.waitboss == value then
			 for num, name in pairs(self.db.profile.Waitlist.members) do
				tinsert(db.raidlog[db.raidnum].bosskills[eventnum].attendees, name)
			end
		else
			self:AwardWaitotimeDKP(self.db.profile.waitboss, POOL, Boss, self.db.profile.Waitlist.members, "Group", "Group")
		end
		self:AwardDKP(nil, self.db.profile.waitboss, POOL, self.db.profile.Waitlist.members, 1, quiet)
	elseif id == 1 then
		if self.db.profile.waitstart == 0 then return end
		if self.db.profile.waitstart == value  then
			 for num, name in pairs(self.db.profile.Waitlist.members) do
				tinsert(db.raidlog[db.raidnum].bosskills[1].attendees, name)
			end
		else
			self:AwardWaitotimeDKP(self.db.profile.waitstart, POOL, Boss, self.db.profile.Waitlist.members, "Group", "Group")
		end
		self:AwardDKP(nil, self.db.profile.waitstart, POOL, self.db.profile.Waitlist.members, 1, quiet)
	end
end

function mod:AwardWaitotimeDKP(value, POOL, eventname, attendees, name, type)
	if not db.raidlog.dkpevents then db.raidlog.dkpevents = { } end
	if not db.raidlog.dkpevents[POOL] then db.raidlog.dkpevents[POOL] = { } end
	table.insert(db.raidlog.dkpevents[POOL], {	name = name,
							value = value,
							members = attendees,
							eqdkp = db.eqDKP[POOL].eqDKPsite,
							prefix = db.eqDKP[POOL].prefix,
							addtype = type,
							event = eventname})
end

function mod:AddIndividualWait(name, POOL)
	local db = self.db.profile
	local id = self:GetEventID(1)
	if db.waitstart > 0 and id then
	 	if db.waitstart == db.raidlog[POOL].bosskills[id].value then
	 		tinsert(db.raidlog[POOL].bosskills[id].attendees, name)
		else self:AwardWaitotimeDKP(db.waitstart, POOL, db.raidlog[POOL].bosskills[id].name, {[1] = name}, name, "individual")
		end
		self:AwardDKP(nil, db.waitstart, POOL, {[1] = name}, 1)
	end
end

function mod:AddWait(addname)
	if UnitInRaid(addname) then return end
	local db = self.db.profile
	local wait = nil
	local POOL = db.raid
	local class = nil
	local ttime = date("%m/%d/%y %H:%M:%S")
	
	--initialise wl
	if not db.Waitlist then db.Waitlist = {members = {}, join = {} } end
	for num, name in pairs(db.Waitlist.members) do
		if name == addname then wait = true break end
	end
	
	--add member
	if not wait then
		table.insert(db.Waitlist.members, addname)
		db.Waitlist.join[addname] = ttime
		self:out(fmt(LL["Added %s to the waitlist."], addname))
		self:PlayerinDB(addname)
		if db.InRaid then self:AddIndividualWait(addname, POOL) end
		
		--update wl
		if mdkp.querytooltips["listdkp"]  then
			local found
			for i, data in pairs(mdkp.querytooltips["listdkp"]) do
				if data[1].n == addname then
					found = i
					break
				end
			end
			local status = mdkp.guildstatustext[L["WAIT"]]
			if not found then
				local namedata, pointsdata = self:GetDKPInfo(addname, POOL)
  				tinsert(mdkp.querytooltips["waitlist"], {namedata, status, pointsdata})
  			else mdkp.querytooltips["listdkp"][found][2] = status end
  			self:ToggleFrameUpdate("listdkp")
		end
	end
end

function mod:SubWait(name)
	local db = self.db.profile
	for num, waitname in pairs(db.Waitlist.members) do
		if name == waitname then
			tremove(db.Waitlist.members, num)
			db.Waitlist.join[name] = nil
			self:out(fmt(LL["Removed %s from wailist."], name))
			break
		end
	end
	
	--update tablet if open
	if mdkp.querytooltips["listdkp"]  then
		for num, v in pairs(mdkp.querytooltips["listdkp"]) do
			if v[1].n == name then
				v[2] = mdkp.guildstatustext[L["ONLINE"]]
				break
			end
		end
		self:ToggleFrameUpdate("listdkp")
	end
end

function mod:OnWaitWhisper(sender, actions)
	actions = string.utf8upper(actions)
	local args = {self:Explode(actions, " ")}
	local action = args[2]
	local member = args[3]
	if member and string.utf8upper(sender) == member then member = nil end
	local POOL = db.raid
	local main = self:CheckforAlias(sender, POOL)
	if action == "WAIT" or action == "WAIT+" then
		if member then
			main = string.utf8upper(sub(member, 1, 1)) .. string.utf8lower(sub(member, 2, -1))
			local core = self:GetModuleRef("CoreModule")
			if core then core:AddAlias(sender, main, true) end
		end
		self:AddWait(main)
		SendChatMessage(LL["Added you to the waitlist."], "WHISPER", nil, sender)
	elseif action == "WAIT-" then
		self:SubWait(main)
		SendChatMessage(LL["Removed you from the waitlist."], "WHISPER", nil, sender)
	end
end

function mod:IsOnWaitlist(onname)
	local wait = nil
	local POOL = db.raid
	for num, name in pairs(self.db.profile.Waitlist.members) do
		if name == onname then wait = true break end
	end
	return wait
end

function mod:InformOnlineAlias(name, onlinemembers)
	local POOL = db.raid
	if onlinemembers[name] and onlinemembers[name].o then return name end
	for alt, main in pairs(db.eqDKP[POOL].Aliases) do
		self:debug(alt.." "..main.." "..name)
		if main == name then
			if onlinemembers[alt] and onlinemembers[alt].o then 
				SendChatMessage(LL["Please log on to your main for an invite to the raid.  Whisper me when online."], "WHISPER", nil, alt)
				name = alt
				break
			end
		end
	end
	return name
end

function mod:AwardWaitlistDKPperTime(POOL, Runname)
	if self.db.profile.waitaward and self.db.profile.waitotime ~= 0 and #self.db.profile.Waitlist.members ~= 0 then
		self:AwardWaitotimeDKP(self.db.profile.waitotime, POOL, Runname, self.db.profile.Waitlist.members, "Group", "Group") 
		self:AwardDKP(nil, self.db.profile.waitotime, POOL, self.db.profile.Waitlist.members, 1)
	end
end